'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _fsExtra = require('fs-extra');

var _fsExtra2 = _interopRequireDefault(_fsExtra);

var _nightmareHandler = require('nightmare-handler');

var _SceenshotConfig = require('./SceenshotConfig');

var _SceenshotConfig2 = _interopRequireDefault(_SceenshotConfig);

var _CoverageConfig = require('./CoverageConfig');

var _CoverageConfig2 = _interopRequireDefault(_CoverageConfig);

var _DeviceConfig = require('./DeviceConfig');

var _DeviceConfig2 = _interopRequireDefault(_DeviceConfig);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var BaseHandle = function () {
    /**
     * 构造函数
     *
     * @param {String} pageUrl 页面的 URL 地址
     * @param {String} crawlerScriptPath 运行在浏览器中的前端爬虫脚本，需要是绝对路径
     * @param {Object} [opts] 额外参数
     * @param {Boolean} [opts.show] 是否需要展示浏览器，默认为 false
     * @param {String} [opts.proxyServer] 代理服务器，例如 127.0.0.1:8899，或者 `my_proxy_server.example.com:8080`，详见 https://github.com/segmentio/nightmare#switches
     * @param {String | Number} [opts.wait] wait配置，会直接透传给 nightmare 的 wait 配置项，详细请查看 https://github.com/segmentio/nightmare#waitms
     * @param {Boolean} [opts.doNotEnd] 是否在执行完成之后不要关闭浏览器，默认为 false
     * @param {String} [opts.cookie] 为浏览器注入cookie，格式与 document.cookie 一致
     * @param {Object} [opts.mockstarQuery] 指定 mockstar 的query参数，用于数据打桩
     * @param {Boolean} [opts.useRecorder] 是否使用记录器记录所有浏览器行为，包括请求等
     * @param {String} [opts.tag] 额外标记，用于区分不同的执行，在截图等场景做区分
     * @param {undefined | ScreenshotConfig} [opts.screenshotConfig] 截图设置
     * @param {undefined | DeviceConfig} [opts.deviceConfig] 设备设置
     * @param {undefined | CoverageConfig} [opts.coverageConfig] 测试覆盖率设置
     */
    function BaseHandle(pageUrl, crawlerScriptPath) {
        var opts = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};
        (0, _classCallCheck3.default)(this, BaseHandle);

        this.pageUrl = pageUrl;
        this.crawlerScriptPath = crawlerScriptPath;

        // 校验 crawlerScriptPath 必须是存在的，否则后续的逻辑也执行不了
        // 如果获取 client script 地址不存在，则抛出异常提示
        if (!_fs2.default.existsSync(crawlerScriptPath)) {
            throw new Error('Unknown crawlerScriptPath=' + crawlerScriptPath);
        }

        this.show = opts.show;
        this.wait = opts.wait;
        this.doNotEnd = opts.doNotEnd;
        this.cookie = opts.cookie;

        /**
         * 设置代理服务器。
         * https://github.com/segmentio/nightmare#switches
         *
         * @type {String} 代理服务器，类似 my_proxy_server.example.com:8080
         */
        this.proxyServer = opts.proxyServer;

        this.mockstarQuery = opts.mockstarQuery || null;

        // 是否使用记录器记录整个请求队列
        // 如果为 true，则可以从 this.globalInfo.recorder 中获取，
        // 如果为 字符串，则可以从 this.globalInfo[xxx] 中获取，
        this.useRecorder = function (useRecorder) {
            if (!useRecorder) {
                return false;
            }

            return typeof useRecorder === 'boolean' ? 'recorder' : useRecorder;
        }(opts.useRecorder);

        // 截屏设置
        this.screenshotConfig = opts.screenshotConfig;

        // 设备设置
        this.deviceConfig = opts.deviceConfig;

        // 测试覆盖率
        this.coverageConfig = opts.coverageConfig;

        this.globalInfo = {};

        this.onNightmareCreated = function (self) {};

        this.onBeforeGotoPage = function (self) {};

        this.onElectronClose = function (self) {};

        this.actionList = [];

        this._dataIndexMap = {};
    }

    (0, _createClass3.default)(BaseHandle, [{
        key: 'getResult',
        value: function () {
            var _ref = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee() {
                var _this = this;

                var nightmareConfig, NightmarePlus, result, i, length, curRun, t, coverageFilePath;
                return _regenerator2.default.wrap(function _callee$(_context) {
                    while (1) {
                        switch (_context.prev = _context.next) {
                            case 0:
                                // nightmare 初始化参数
                                nightmareConfig = {
                                    show: this.show,
                                    webPreferences: {
                                        // 用例过多且频繁启动测试时可能会存在失败的场景 #154
                                        partition: 'nopersist',
                                        preload: this.crawlerScriptPath
                                    }
                                };


                                if (nightmareConfig.show) {
                                    // https://www.npmjs.com/package/nightmare#opendevtools
                                    nightmareConfig.openDevTools = {
                                        mode: 'detach'
                                    };
                                }

                                // 设置代理服务器
                                if (this.proxyServer) {
                                    nightmareConfig.switches = {
                                        'proxy-server': this.proxyServer,

                                        // 必须设置一下这个，否则在某些情况下 https 地址无法使用
                                        // https://github.com/matmanjs/matman/issues/159
                                        'ignore-certificate-errors': true
                                    };
                                }

                                // 如果有设置符合要求的 matman 服务设置，则还需要额外处理一下
                                if (this.mockstarQuery && typeof this.mockstarQuery.getQueryString === 'function') {
                                    this.pageUrl = this.pageUrl + (this.pageUrl.indexOf('?') > -1 ? '&' : '?') + this.mockstarQuery.getQueryString();
                                }

                                // console.log('===nightmareConfig====', nightmareConfig);

                                // 创建 nightmare 对象
                                NightmarePlus = (0, _nightmareHandler.getNightmarePlus)();

                                this.nightmare = NightmarePlus(nightmareConfig);

                                // 创建完成之后，可能会有一些自己的处理
                                this.onNightmareCreated(this);

                                // 使用记录器
                                if (this.useRecorder) {
                                    this.globalInfo[this.useRecorder] = new _nightmareHandler.WebEventRecorder(this.nightmare);
                                }

                                // 初始化一些行为
                                this.nightmareRun = this.nightmare.header('x-mat-from', 'nightmare').header('x-mat-timestamp', Date.now());

                                // 设置设备
                                if (this.deviceConfig) {
                                    this.nightmareRun = this.nightmareRun.exDevice(this.deviceConfig.name, {
                                        UA: this.deviceConfig.UA,
                                        width: this.deviceConfig.width,
                                        height: this.deviceConfig.height
                                    });
                                }

                                // 设置 cookie
                                if (this.cookie) {
                                    this.nightmareRun = this.nightmareRun.exCookies(this.cookie, getMainUrl(this.pageUrl));
                                }

                                // 加载页面之前要执行的方法
                                this.onBeforeGotoPage(this);

                                // 加载页面
                                this.nightmareRun = this.nightmareRun.goto(this.pageUrl);

                                // 如果指定了 opts.wait，则会传递给 nightmare 处理，具体使用方法可以参考：
                                // https://github.com/segmentio/nightmare#waitms
                                if (typeof this.wait !== 'undefined') {
                                    this.nightmareRun = this.nightmareRun.wait(this.wait);
                                }

                                // 循环处理多个 action
                                result = [];
                                i = 0, length = this.actionList.length;

                            case 16:
                                if (!(i < length)) {
                                    _context.next = 37;
                                    break;
                                }

                                curRun = this.actionList[i](this.nightmareRun);


                                if (this.screenshotConfig) {
                                    curRun.screenshot(this.screenshotConfig.getPathWithId(i + 1), this.screenshotConfig.clip);
                                }

                                _context.next = 21;
                                return curRun.evaluate(evaluate);

                            case 21:
                                t = _context.sent;

                                if (!(t.__coverage__ && this.coverageConfig)) {
                                    _context.next = 33;
                                    break;
                                }

                                coverageFilePath = this.coverageConfig.getPathWithId(i + 1);
                                _context.prev = 24;
                                _context.next = 27;
                                return _fsExtra2.default.outputJson(coverageFilePath, t.__coverage__);

                            case 27:

                                // 记录之后就删除之
                                delete t.__coverage__;
                                _context.next = 33;
                                break;

                            case 30:
                                _context.prev = 30;
                                _context.t0 = _context['catch'](24);

                                console.log('save coverage file fail', coverageFilePath, _context.t0);

                            case 33:

                                result.push(t);

                            case 34:
                                i++;
                                _context.next = 16;
                                break;

                            case 37:

                                // 暴露 electron 关闭时间，注意它在 nightmareRun.end() 之后才会触发
                                if (typeof this.onElectronClose === 'function' && this.nightmareRun.proc && this.nightmareRun.proc.on) {
                                    this.nightmareRun.proc.on('close', function (code) {
                                        // var code = {
                                        //     127: 'command not found - you may not have electron installed correctly',
                                        //     126: 'permission problem or command is not an executable - you may not have all the necessary dependencies for electron',
                                        //     1: 'general error - you may need xvfb',
                                        //     0: 'success!'
                                        // }
                                        _this.onElectronClose(code);
                                    });
                                }

                                // 不关闭界面

                                if (!this.doNotEnd) {
                                    _context.next = 43;
                                    break;
                                }

                                _context.next = 41;
                                return this.nightmareRun;

                            case 41:
                                _context.next = 45;
                                break;

                            case 43:
                                _context.next = 45;
                                return this.nightmareRun.end();

                            case 45:
                                return _context.abrupt('return', {
                                    data: result,
                                    _dataIndexMap: this._dataIndexMap,
                                    globalInfo: this.globalInfo
                                });

                            case 46:
                            case 'end':
                                return _context.stop();
                        }
                    }
                }, _callee, this, [[24, 30]]);
            }));

            function getResult() {
                return _ref.apply(this, arguments);
            }

            return getResult;
        }()
    }, {
        key: 'addAction',
        value: function addAction(actionName, actionCall) {
            if (typeof actionCall === 'function') {
                this.actionList.push(actionCall);
                this._dataIndexMap[actionName + ''] = this.actionList.length - 1;
            } else if (typeof actionName === 'function') {
                this.actionList.push(actionName);
            } else {
                throw new Error('addAction should assign function!');
            }
        }

        //
        // onNightmareCreated(callback) {
        //     if (typeof callback === 'function') {
        //         return callback(this.nightmare);
        //     }
        //
        //     return this.nightmare;
        // }

    }]);
    return BaseHandle;
}();

exports.default = BaseHandle;


function evaluate() {
    // 如果没有这个变量，说明注入代码失败
    if (!window.matman_ver) {
        return {
            error: 'preload failed!'
        };
    }

    // window.getPageInfo 必须是个函数
    // window.getPageInfo 方法和其他变量均由 preload 配置中的 js 文件引入
    if (typeof window.getPageInfo !== 'function') {
        return {
            error: 'window.getPageInfo is not function!'
        };
    }

    // 如果存在需要前端执行的代码，则在所有逻辑开始之前执行
    if (window.evalList && window.evalList.length) {
        window.evalList.forEach(function (item) {
            eval(window[item]);
        });
    }

    var pageInfo = window.getPageInfo();

    if (window.__coverage__ && pageInfo) {
        pageInfo.__coverage__ = window.__coverage__;
    }

    return pageInfo;
}

function getMainUrl(url) {
    var arr1 = url.split('//');

    if (arr1.length > 1) {
        var arr2 = arr1[1].split('/');
        return arr1[0] + '//' + arr2[0];
    } else {
        var _arr = arr1[0].split('/');
        return _arr[0];
    }
}
//# sourceMappingURL=BaseHandle.js.map