'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _typeof2 = require('babel-runtime/helpers/typeof');

var _typeof3 = _interopRequireDefault(_typeof2);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _fsExtra = require('fs-extra');

var _fsExtra2 = _interopRequireDefault(_fsExtra);

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _util = require('../util');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var SceenshotConfig = function () {
    /**
     * 构造函数
     *
     * https://github.com/segmentio/nightmare#screenshotpath-clip
     *
     * @param {String | Boolean | Object} opts 是否启用截图，或者截图保存的文件路径，或者截图配置
     * @param {String} [opts.path] 截图保存的完成文件名，如果不填写，则将根据当前路径自动生成名字
     * @param {String} [opts.clip] 截图的区域
     * @param {String} basePath 测试用例的脚本目录
     * @param {String} [tag] 标记
     */
    function SceenshotConfig(opts, basePath, tag) {
        (0, _classCallCheck3.default)(this, SceenshotConfig);

        this.tag = tag;

        // 如果 this.tag 是存在的文件，则获取文件名
        if (_fs2.default.existsSync(this.tag)) {
            this.tag = _path2.default.basename(this.tag).replace(/\./gi, '_');
        }

        if (opts && (typeof opts === 'undefined' ? 'undefined' : (0, _typeof3.default)(opts)) === 'object') {
            this.path = opts.path;
            this.clip = opts.clip;
        } else if (typeof opts === 'string') {
            this.path = this._getPath(opts, basePath);
        } else {
            this.path = this._getPath(undefined, basePath);
        }
    }

    (0, _createClass3.default)(SceenshotConfig, [{
        key: 'getPathWithId',
        value: function getPathWithId(id) {
            return this.path.replace(/(.*)\.(.*)/, function (match, p1, p2) {
                return [p1, id + '.' + p2].join('_');
            });
        }
    }, {
        key: '_getPath',
        value: function _getPath(paramsPath, basePath) {
            // 如果传递了path，则直接使用 path
            // TODO 需要支持非绝对路径
            if (paramsPath) {
                return paramsPath;
            }

            // 根据配置内容获得 crawlerParser 的对象
            var crawlerParser = (0, _util.findCrawlerParser)(basePath);

            // 有可能找不到
            if (!crawlerParser) {
                console.error('Can not find crawlerParser by basePath', basePath);
                return '';
            }

            // 相对路径
            var relativePath = _path2.default.relative(crawlerParser.testerPath, basePath);
            var arr = relativePath.split(_path2.default.sep);

            // 文件名
            var folderName = arr.pop();

            if (this.tag) {
                folderName = folderName + '_' + this.tag;
            }

            // 需要保存的文件夹路径
            var saveDir = _path2.default.join(crawlerParser.screenshotPath, (0, _util.getFolderNameFromPath)(arr.join(_path2.default.sep)));

            // 要保证这个目录存在，否则保存时会报错
            _fsExtra2.default.ensureDirSync(saveDir);

            return _path2.default.join(saveDir, folderName + '.png');
        }
    }]);
    return SceenshotConfig;
}();

exports.default = SceenshotConfig;
//# sourceMappingURL=SceenshotConfig.js.map