'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.isURLMatch = isURLMatch;

var _url = require('url');

/**
 * 匹配 url 是否被部分匹配到
 *
 * @param {String} URLToCheck 待匹配的地址
 * @param {String} partialURL 部分地址
 * @param {Object} query 地址中携带的查询参数
 * @return {Boolean}
 */
function isURLMatch(URLToCheck, partialURL) {
    var query = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};

    if (!URLToCheck || !partialURL) {
        return false;
    }

    // ? 之前的部分一定要能够先匹配到
    if (URLToCheck.split('?')[0].indexOf(partialURL.split('?')[0]) < 0) {
        return false;
    }

    var URLToCheckParseQueryResult = parseQuery((0, _url.parse)(URLToCheck).search);
    var partialURLParseQueryResult = parseQuery((0, _url.parse)(partialURL).search);

    // 合并，因为有可能 url 中也携带了 query 参数
    var actualQuery = Object.assign({}, partialURLParseQueryResult, query);

    // 所有的 query key 值列表
    var queryKeyArr = Object.keys(actualQuery);

    // 只要找到存在一个需要查询的参数与目标不一样，则返回不匹配
    for (var i = 0, length = queryKeyArr.length; i < length; i++) {
        var curKey = queryKeyArr[i];

        if (!isEqualOfLoose(URLToCheckParseQueryResult[curKey], actualQuery[curKey])) {
            return false;
        }
    }

    return true;
}

function parseQuery(url) {
    if (!url) {
        return {};
    }

    var queryObj = {};
    var reg = /[?&]([^=&#]+)=([^&#]*)/g;
    var querys = url.match(reg);

    if (querys) {
        for (var i in querys) {
            var query = querys[i].split('=');
            var key = query[0].substr(1);
            var value = query[1];

            queryObj[key] ? queryObj[key] = [].concat(queryObj[key], value) : queryObj[key] = value;
        }
    }

    return queryObj;
}

function isEqualOfLoose(a, b) {
    return a == b;
}
//# sourceMappingURL=url.js.map